@@ -1,4 +1,4 @@
-# $Id: Session.pm,v 1.64 2001/07/02 04:42:25 rcaputo Exp $
+# $Id: Session.pm,v 1.69 2001/08/11 22:40:05 rcaputo Exp $
 
 package POE::Session;
 
@@ -8,32 +8,34 @@ use POSIX qw(ENOSYS);
 
 use POE::Preprocessor;
 
-enum SE_NAMESPACE SE_OPTIONS SE_STATES
+sub SE_NAMESPACE    () { 0 }
+sub SE_OPTIONS      () { 1 }
+sub SE_STATES       () { 2 }
 
-# I had made these constant subs, but you can't use constant subs as
-# hash keys, so they're POE::Preprocessor constants.  Blargh!
+sub CREATE_ARGS     () { 'args' }
+sub CREATE_OPTIONS  () { 'options' }
+sub CREATE_INLINES  () { 'inline_states' }
+sub CREATE_PACKAGES () { 'package_states' }
+sub CREATE_OBJECTS  () { 'object_states' }
+sub CREATE_HEAP     () { 'heap' }
 
-const CREATE_ARGS     'args'
-const CREATE_OPTIONS  'options'
-const CREATE_INLINES  'inline_states'
-const CREATE_PACKAGES 'package_states'
-const CREATE_OBJECTS  'object_states'
-const CREATE_HEAP     'heap'
+sub OPT_TRACE       () { 'trace' }
+sub OPT_DEBUG       () { 'debug' }
+sub OPT_DEFAULT     () { 'default' }
 
-const OPT_TRACE   'trace'
-const OPT_DEBUG   'debug'
-const OPT_DEFAULT 'default'
-
-const EN_START   '_start'
-const EN_DEFAULT '_default'
-
-# Define some debugging flags for subsystems, unless someone already
-# has defined them.
-BEGIN {
-  defined &DEB_DESTROY or eval 'sub DEB_DESTROY () { 0 }';
-}
+sub EN_START        () { '_start' }
+sub EN_DEFAULT      () { '_default' }
 
 #------------------------------------------------------------------------------
+# Macros.
+
+macro define_assert (<const>) {
+  defined &ASSERT_<const> or eval 'sub ASSERT_<const> () { ASSERT_DEFAULT }';
+}
+
+macro define_trace (<const>) {
+  defined &TRACE_<const> or eval 'sub TRACE_<const> () { TRACE_DEFAULT }';
+}
 
 macro make_session {
   my $self =
@@ -41,6 +43,9 @@ macro make_session {
             { }, # SE_OPTIONS
             { }, # SE_STATES
           ], $type;
+  if (ASSERT_STATES) {
+    $self->[SE_OPTIONS]->{+OPT_DEFAULT} = 1;
+  }
 }
 
 macro validate_kernel {
@@ -50,7 +55,7 @@ macro validate_kernel {
 
 macro validate_state {
   carp "redefining state($name) for session(", {% fetch_id $self %}, ")"
-    if ( $self->[SE_OPTIONS]->{OPT_DEBUG} &&
+    if ( $self->[SE_OPTIONS]->{+OPT_DEBUG} &&
          (exists $self->[SE_STATES]->{$name})
        );
 }
@@ -64,7 +69,7 @@ macro verify_start_state {
   # do we know what to do?  Don't even bother registering the session
   # if the start state doesn't exist.
 
-  if (exists $self->[SE_STATES]->{EN_START}) {
+  if (exists $self->[SE_STATES]->{+EN_START}) {
     $POE::Kernel::poe_kernel->session_alloc($self, @args);
   }
   else {
@@ -75,9 +80,55 @@ macro verify_start_state {
 
 # MACROS END <-- search tag for editing
 
+#------------------------------------------------------------------------------
+# Debugging flags for subsystems.  They're done as double evals here
+# so that someone may define them before using POE::Session (or POE),
+# and the pre-defined value will take precedence over the defaults
+# here.
+
+# Define some debugging flags for subsystems, unless someone already
+# has defined them.
+BEGIN {
+  defined &DEB_DESTROY or eval 'sub DEB_DESTROY () { 0 }';
+}
+
+
+BEGIN {
+
+  # ASSERT_DEFAULT changes the default value for other ASSERT_*
+  # constants.  It inherits POE::Kernel's ASSERT_DEFAULT value, if
+  # it's present.
+
+  unless (defined &ASSERT_DEFAULT) {
+    if (defined &POE::Kernel::ASSERT_DEFAULT) {
+      eval( "sub ASSERT_DEFAULT () { " . &POE::Kernel::ASSERT_DEFAULT . " }" );
+    }
+    else {
+      eval 'sub ASSERT_DEFAULT () { 0 }';
+    }
+  };
+
+  # TRACE_DEFAULT changes the default value for other TRACE_*
+  # constants.  It inherits POE::Kernel's TRACE_DEFAULT value, if
+  # it's present.
+
+  unless (defined &TRACE_DEFAULT) {
+    if (defined &POE::Kernel::TRACE_DEFAULT) {
+      eval( "sub TRACE_DEFAULT () { " . &POE::Kernel::TRACE_DEFAULT . " }" );
+    }
+    else {
+      eval 'sub TRACE_DEFAULT () { 0 }';
+    }
+  };
+
+  {% define_assert STATES  %}
+  {% define_trace  DESTROY %}
+}
+
 #------------------------------------------------------------------------------
 # Export constants into calling packages.  This is evil; perhaps
-# EXPORT_OK instead?
+# EXPORT_OK instead?  The parameters NFA has in common with SESSION
+# (and other sessions) must be kept at the same offsets as each-other.
 
 sub OBJECT  () {  0 }
 sub SESSION () {  1 }
@@ -85,16 +136,17 @@ sub KERNEL  () {  2 }
 sub HEAP    () {  3 }
 sub STATE   () {  4 }
 sub SENDER  () {  5 }
-sub ARG0    () {  6 }
-sub ARG1    () {  7 }
-sub ARG2    () {  8 }
-sub ARG3    () {  9 }
-sub ARG4    () { 10 }
-sub ARG5    () { 11 }
-sub ARG6    () { 12 }
-sub ARG7    () { 13 }
-sub ARG8    () { 14 }
-sub ARG9    () { 15 }
+# NFA keeps its state in 6.  unused in session so that args match up.
+sub ARG0    () {  7 }
+sub ARG1    () {  8 }
+sub ARG2    () {  9 }
+sub ARG3    () { 10 }
+sub ARG4    () { 11 }
+sub ARG5    () { 12 }
+sub ARG6    () { 13 }
+sub ARG7    () { 14 }
+sub ARG8    () { 15 }
+sub ARG9    () { 16 }
 
 sub import {
   my $package = caller();
@@ -118,7 +170,7 @@ sub import {
 }
 
 #------------------------------------------------------------------------------
-# Classic style constructor.  This is unofficially depreciated in
+# Classic style constructor.  This is unofficially deprecated in
 # favor of the create() constructor.  Its DWIM nature does things
 # people don't mean, so create() is a little more explicit.
 
@@ -163,7 +215,7 @@ sub new {
       # Check for common problems.
 
       unless ((defined $first) && (length $first)) {
-        carp "depreciated: using an undefined state name";
+        carp "deprecated: using an undefined state name";
       }
 
       if (ref($first) eq 'CODE') {
@@ -304,26 +356,26 @@ sub create {
   # we're given.  If the arguments are a list reference, map its items
   # to ARG0..ARGn; otherwise make whatever the heck it is be ARG0.
 
-  if (exists $params{CREATE_ARGS}) {
-    if (ref($params{CREATE_ARGS}) eq 'ARRAY') {
-      push @args, @{$params{CREATE_ARGS}};
+  if (exists $params{+CREATE_ARGS}) {
+    if (ref($params{+CREATE_ARGS}) eq 'ARRAY') {
+      push @args, @{$params{+CREATE_ARGS}};
     }
     else {
-      push @args, $params{CREATE_ARGS};
+      push @args, $params{+CREATE_ARGS};
     }
-    delete $params{CREATE_ARGS};
+    delete $params{+CREATE_ARGS};
   }
 
   # Process session options here.  Several options may be set.
 
-  if (exists $params{CREATE_OPTIONS}) {
-    if (ref($params{CREATE_OPTIONS}) eq 'HASH') {
-      $self->[SE_OPTIONS] = $params{CREATE_OPTIONS};
+  if (exists $params{+CREATE_OPTIONS}) {
+    if (ref($params{+CREATE_OPTIONS}) eq 'HASH') {
+      $self->[SE_OPTIONS] = $params{+CREATE_OPTIONS};
     }
     else {
       croak "options for $type constructor is expected to be a HASH reference";
     }
-    delete $params{CREATE_OPTIONS};
+    delete $params{+CREATE_OPTIONS};
   }
 
   # Get down to the business of defining states.
@@ -381,7 +433,9 @@ sub create {
         }
 
         else {
-          croak "states for '$package' needs to be a hash or array ref";
+          croak( "states for package '$package' " .
+                 "need to be a hash or array ref"
+               );
         }
       }
     }
@@ -425,7 +479,7 @@ sub create {
         }
 
         else {
-          croak "states for '$object' needs to be a hash or array ref";
+          croak "states for object '$object' need to be a hash or array ref";
         }
 
       }
@@ -481,7 +535,7 @@ sub _invoke_state {
 
   # Trace the state invocation if tracing is enabled.
 
-  if ($self->[SE_OPTIONS]->{OPT_TRACE}) {
+  if ($self->[SE_OPTIONS]->{+OPT_TRACE}) {
     warn {% fetch_id $self %}, " -> $state (from $file at $line)\n";
   }
 
@@ -494,9 +548,9 @@ sub _invoke_state {
     # Drop the state transition event on the floor, and optionally
     # make some noise about it.
 
-    unless (exists $self->[SE_STATES]->{EN_DEFAULT}) {
+    unless (exists $self->[SE_STATES]->{+EN_DEFAULT}) {
       $! = ENOSYS;
-      if ($self->[SE_OPTIONS]->{OPT_DEFAULT}) {
+      if ($self->[SE_OPTIONS]->{+OPT_DEFAULT}) {
         warn( "a '$state' state was sent from $file at $line to session ",
               {% fetch_id $self %}, ", but session ", {% fetch_id $self %},
               " has neither that state nor a _default state to handle it\n"
@@ -508,7 +562,7 @@ sub _invoke_state {
     # If we get this far, then there's a _default state to redirect
     # the transition to.  Trace the redirection.
 
-    if ($self->[SE_OPTIONS]->{OPT_TRACE}) {
+    if ($self->[SE_OPTIONS]->{+OPT_TRACE}) {
       warn {% fetch_id $self %}, " -> $state redirected to _default\n";
     }
 
@@ -532,6 +586,7 @@ sub _invoke_state {
         $self->[SE_NAMESPACE],          # heap
         $state,                         # state
         $source_session,                # sender
+        undef,                          # unused #6
         @$etc                           # args
       );
   }
@@ -546,6 +601,7 @@ sub _invoke_state {
         $self->[SE_NAMESPACE],          # heap
         $state,                         # state
         $source_session,                # sender
+        undef,                          # unused #6
         @$etc                           # args
       );
 }
@@ -582,7 +638,7 @@ sub register_state {
 
     else {
       if ( (ref($handler) eq 'CODE') and
-           $self->[SE_OPTIONS]->{OPT_TRACE}
+           $self->[SE_OPTIONS]->{+OPT_TRACE}
          ) {
         carp( {% fetch_id $self %},
               " : state($name) is not a proper ref - not registered"
@@ -1659,6 +1715,32 @@ session.  The existing session which created a child becomes its
 A session with children will not spontaneously stop.  In other words,
 the presence of child sessions will keep a parent alive.
 
+=head2 Session's Debugging Features
+
+POE::Session contains a two debugging assertions, for now.
+
+=over 2
+
+=item ASSERT_DEFAULT
+
+ASSERT_DEFAULT is used as the default value for all the other assert
+constants.  Setting it true is a quick and reliably way to ensure all
+Session assertions are enabled.
+
+Session's ASSERT_DEFAULT inherits Kernel's ASSERT_DEFAULT value unless
+overridden.
+
+=item ASSERT_STATES
+
+Setting ASSERT_STATES to true causes every Session to warn when they
+are asked to handle unknown events.  Session.pm implements the guts of
+ASSERT_STATES by defaulting the "default" option to true instead of
+false.  See the option() function earlier in this document for details
+about the "default" option.
+
+=back
+
+
 =head1 SEE ALSO
 
 POE::Kernel.
